import arelle
import time
from arelle import ViewFileFactList, ModelXbrl, ModelManager, FileSource, Cntlr, ModelDocument
import json
import os


def extractXbrl(path):
    # uses the black box that is arele (available on pip, it is a modified version done by Przemek and Rémi)
    modelManager = arelle.ModelManager.initialize(arelle.Cntlr.Cntlr())
    xbrl = arelle.ModelXbrl.load(modelManager, path)
    jsonobject = arelle.ViewFileFactList.viewFacts(xbrl, 'xbrlDocument.json')
    return jsonobject


def run_test(dir_with_samples):
    print("Running tests inside {}".format(dir_with_samples))
    for file in os.listdir(dir_with_samples):
        if not file.endswith('.xhtml'):
            continue
        path = os.path.join(dir_with_samples, file)

        # original code, without modifications
        ModelDocument.AWESOME_MODS = False
        st = time.time()
        org_raw = extractXbrl(path)
        org = json.dumps(org_raw)
        end = time.time()

        # turn on optimizations
        ModelDocument.AWESOME_MODS = True
        st2 = time.time()
        quick_raw = extractXbrl(path)
        quick = json.dumps(quick_raw)
        end2 = time.time()
        print("[{}] XBLR load: {:.3f}s vs. {:.3f}s".format(os.path.basename(path), end - st, end2 - st2))
        # if org != quick:
        #     print("Org: {}".format(org))
        #     print("Qck: {}\n".format(quick))

        filename = os.path.basename(path).split('.')[0]
        with open("out_{}_org.json".format(filename), 'wt') as f:
            json.dump(org_raw, f, indent=4)

        with open("out_{}_qck.json".format(filename), 'wt') as f:
            json.dump(quick_raw, f, indent=4)


if __name__ == "__main__":
    run_test('samples')
    exit(0)
    path = 'samples\\test01.xhtml'
    # original code, without modifications
    ModelDocument.AWESOME_MODS = False
    st = time.time()
    org = json.dumps(extractXbrl(path))
    end = time.time()

    # turn on optimizations
    ModelDocument.AWESOME_MODS = True
    st2 = time.time()
    quick = json.dumps(extractXbrl(path))
    end2 = time.time()
    print("[{}] XBLR load: {:.3f}s vs. {:.3f}s".format(os.path.basename(path), end - st, end2 - st2))
    if org != quick:
        print("Org: {}".format(org))
        print("Qck: {}\n".format(quick))
    print(org)
    print(quick)


import os
import requests
from multiprocessing import Pool
import time
import json
import TestArelle

# set where to save files
CACHE_DIR = 'cache'

# arelle config dir (contains edgartaxonomies-all-years.xml, hmrc-taxonomies.xml etc.)
#XMLS_DIR = r'C:\Lendflo\Arelle-master\arelle\config'
XMLS_DIR = r'/home/ubuntu/env/lib/python3.5/site-packages/arelle/config'
# dir with XBRL samples
XBLR_SAMPLES_DIR = 'samples'


def get_file(url):
    locations = url.split('://')
    assert len(locations) == 2, "Tu ma być 2 kuuurłaaaa"
    dest = os.path.join(CACHE_DIR, os.sep.join(locations).replace('/', os.sep))
    dirs = os.path.dirname(dest)
    if not os.path.exists(dirs):
        os.makedirs(dirs)

    if os.path.exists(dest):
        return True

    print("Downloading: {}".format(url))
    try:
        r = requests.get(url, stream=True)
        if r.status_code == 200:
            with open(dest, 'wb') as f:
                for chunk in r:
                    f.write(chunk)
                return True
        else:
            print("File {} is unavailable".format(url))
    except Exception as ex:
        print("Exception: {}".format(str(ex)))

    return False


def extract_links(path_to_dir_with_xmls):
    results = []
    for item in os.listdir(path_to_dir_with_xmls):
        if not item.endswith('.xml'):
            continue

        print("Processing {}".format(item))
        with open(os.path.join(path_to_dir_with_xmls, item), 'rt') as f:
            for line in f:
                start_index = line.find('>http')
                while start_index != -1:
                    end_index = line.find('.xsd<', start_index+1)
                    if end_index == -1:
                        end_index = line.find('.xml<', start_index+1)
                        if end_index == -1:
                            break
                    url = line[start_index + 1:end_index + 4]
                    if url not in results:
                        results.append(url)
                    start_index = line.find('>http', start_index+1)
    return results


if __name__ == "__main__":

    # find links in schema templates
    links = extract_links(XMLS_DIR)
    print("Found {} links".format(len(links)))

    if len(links) == 0:
        print("Something went wrong! Make sure you set correct XMLS_DIR AND CACHE_DIR")
        exit(1)

    # download all
    with Pool() as pool:
        results = pool.map(get_file, links)

    # now prepare .json file with cached links
    timeNow = time.time()
    timeNowStr = time.strftime('%Y-%m-%dT%H:%M:%S UTC', time.gmtime(timeNow))

    xlinks = {}
    for index, value in enumerate(links):
        if results[index]:
            xlinks[value] = timeNowStr

    file = os.path.join(os.path.dirname(os.path.abspath(CACHE_DIR)), "cachedUrlCheckTimes.json")
    print("Saving JSON file to: {}".format(file))
    with open(file, 'wt') as f:
        json.dump(xlinks, f, indent=0)

    # run tests which will also download additional caches
    TestArelle.run_test(XBLR_SAMPLES_DIR)

    print("\nDone. Auf Wiedersehen!")



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="arelle",
    version="2.0",
    author="Przemyslaw Winszczyk, Filip Duch, Remi Tuyaerts",
    description="Light arelle package to extract xbrl files",
    url="https://github.com/LendFlo/Lendflo-arelle",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)